@@ -25,7 +25,7 @@ def route_table(stem, disable_bgp_route_propagation=None, depends_on=None):
 
 def route_to_internet(stem, route_table_name):
     r_i = network.Route(
-        f'r-{stem}-i',
+        f'r-{stem}',
         route_name='FirewallDefaultRoute',  # name required
         resource_group_name=resource_group_name,
         address_prefix='0.0.0.0/0',
